<?php

declare(strict_types=1);

namespace Drupal\conditions_field\Plugin\Field\FieldType;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\json_field\Plugin\Field\FieldType\NativeJsonItem;

/**
 * Defines the 'conditions' field type.
 *
 * @FieldType(
 *   id = "conditions",
 *   label = @Translation("Conditions"),
 *   description = @Translation("Stores condition configurations as json."),
 *   category = "data",
 *   default_widget = "conditions_groups",
 *   default_formatter = "conditions_empty_formatter",
 *   constraints = {"valid_json" = {}}
 * )
 */
class ConditionsItem extends NativeJsonItem {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = parent::schema($field_definition);
    $schema['columns']['condition_logic'] = [
      'type' => 'varchar',
      'length' => 255,
      'binary' => FALSE,
    ];
    $schema['columns']['group_condition_logic'] = [
      'type' => 'varchar',
      'length' => 255,
      'binary' => FALSE,
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('value')->getValue();
    return empty($value) || empty(JSON::decode($value));
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = parent::propertyDefinitions($field_definition);
    $properties['condition_logic'] = DataDefinition::create('string')
      ->setLabel(t('The condition logic'));
    $properties['group_condition_logic'] = DataDefinition::create('string')
      ->setLabel(t('The group condition logic'));

    return $properties;
  }

  public function getOperator(): string {
    return $this->getValue()['group_condition_logic'] ?? 'and';
  }

}
